#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2020 LG Electronics Inc.
# SPDX-License-Identifier: GPL-3.0-only
import os
import logging
import platform
from datetime import datetime
from pathlib import Path
from yaml import safe_dump
from fosslight_reuse._help import print_help_msg
from fosslight_util.constant import LOGGER_NAME
from fosslight_util.set_log import init_log
from ._parsing_yaml import convert_yml_to_excel
from ._parsing_excel import convert_excel_to_yaml

_PKG_NAME = "fosslight_reuse"
logger = logging.getLogger(LOGGER_NAME)


def find_report_file(path_to_find):
    file_to_find = ["FOSSLight-Report.xlsx", "OSS-Report.xlsx"]

    try:
        for file in file_to_find:
            file_with_path = os.path.join(path_to_find, file)
            if os.path.isfile(file_with_path):
                return file
        for root, dirs, files in os.walk(path_to_find):
            for file in files:
                file_name = file.lower()
                if (file_name.startswith("oss-report") or file_name.startswith("fosslight-report")) \
                   and file_name.endswith(".xlsx"):
                    return os.path.join(root, file)
    except Exception as error:
        logger.debug("Find report:"+str(error))
    return ""


def convert_report(base_path, files, output_name):
    oss_pkg_files = ["oss-pkg-info.yml", "oss-pkg-info.yaml"]
    file_option_on = False
    convert_yml_mode = False
    convert_excel_mode = False
    report_to_read = ""
    now = datetime.now().strftime('%Y%m%d_%H-%M-%S')
    is_window = platform.system() == "Windows"

    if output_name == "":
        output_report = "FOSSLight-Report_" + now
        output_yaml = "oss-pkg-info_" + now
        output_dir = os.getcwd()
    else:
        output_report = output_name
        output_yaml = output_name
        try:
            output_dir = os.path.dirname(output_name)
            Path(output_dir).mkdir(parents=True, exist_ok=True)
        except Exception:
            pass

    logger, _result_log = init_log(os.path.join(output_dir, "fosslight_reuse_log_"+now+".txt"),
                                   True, logging.INFO, logging.DEBUG, _PKG_NAME, base_path)

    if base_path != "":
        convert_yml_mode = True

    if files != "":
        if files.endswith(".xlsx"):
            convert_excel_mode = True
            report_to_read = files
        else:
            oss_pkg_files = files.split(',')
            convert_yml_mode = True
            file_option_on = True

    if not convert_yml_mode and not convert_excel_mode:
        if is_window:
            convert_yml_mode = True
            base_path = os.getcwd()
            report_to_read = find_report_file(base_path)
            if report_to_read != "":
                convert_excel_mode = True
        else:
            print_help_msg()

    if convert_yml_mode:
        convert_yml_to_excel(oss_pkg_files, output_report, file_option_on, base_path, is_window)

    if convert_excel_mode:
        convert_excel_to_yaml(report_to_read, output_yaml)

    try:
        _str_final_result_log = safe_dump(_result_log, allow_unicode=True, sort_keys=True)
        logger.info(_str_final_result_log)
    except Exception as ex:
        logger.warning("Failed to print result log. " + str(ex))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: GPL-3.0-only

import logging
from fosslight_util.constant import LOGGER_NAME

_logger = logging.getLogger(LOGGER_NAME)


class OssItem:
    name = "-"
    version = ""
    licenses = []
    source = ""
    files = []
    copyright = ""
    comment = ""
    exclude = ""
    homepage = ""
    relative_path = ""

    def __init__(self, value):
        self.name = "-"
        self.version = ""
        self.licenses = []
        self.source = ""
        self.files = []
        self.copyright = ""
        self.comment = ""
        self.exclude = ""
        self.homepage = ""
        self.relative_path = value

    def __del__(self):
        pass

    def set_homepage(self, value):
        self.homepage = value

    def set_comment(self, value):
        self.comment = value

    def set_copyright(self, value):
        if value != "":
            value = value.strip()
        self.copyright = value

    def set_exclude(self, value):
        if value:
            self.exclude = "Exclude"
        else:
            self.exclude = ""

    def set_name(self, value):
        self.name = value

    def set_version(self, value):
        self.version = str(value)

    def set_licenses(self, value):
        if not isinstance(value, list):
            value = value.split(",")
        self.licenses.extend(value)
        self.licenses = [item.strip() for item in self.licenses]
        self.licenses = list(set(self.licenses))

    def set_files(self, value):
        if isinstance(value, list):
            self.files.extend(value)
        else:
            self.files.append(value)
        self.files = list(set(self.files))

    def set_source(self, value):
        self.source = value

    def get_print_array(self):
        items = []
        if len(self.files) == 0:
            self.files.append("")
        if len(self.licenses) == 0:
            self.licenses.append("")
        for file in self.files:
            lic = ",".join(self.licenses)
            if self.relative_path != "" and not str(self.relative_path).endswith("/"):
                self.relative_path += "/"
            items.append([self.relative_path + file, self.name, self.version, lic, self.source, self.homepage,
                          self.copyright, "", self.exclude, self.comment])
        return items

    def get_print_json(self):
        json_item = {}
        json_item["name"] = self.name

        if self.version != "":
            json_item["version"] = self.version
        if self.source != "":
            json_item["source"] = self.source
        if self.copyright != "":
            json_item["copyright"] = self.copyright
        if self.exclude != "":
            json_item["exclude"] = self.exclude
        if self.comment != "":
            json_item["comment"] = self.comment
        if self.homepage != "":
            json_item["homepage"] = self.homepage

        if len(self.files) > 0:
            json_item["file"] = self.files
        if len(self.licenses) > 0:
            json_item["license"] = self.licenses

        return json_item


def invalid(cmd):
    _logger.info('[{}] is invalid'.format(cmd))
